/*-
 * rmt.c --
 *	Functions to handle the exportation of targets using the
 *	customs daemon. Should make use of Customs_Ping() to make
 *	sure the agent is alive at the beginning...
 *
 * Copyright (c) 1987 by the Regents of the University of California
 * Copyright (c) 1987 by Adam de Boor, UC Berkeley
 *
 * Permission to use, copy, modify, and distribute this
 * software and its documentation for any purpose and without
 * fee is hereby granted, provided that the above copyright
 * notice appear in all copies.  The University of California
 * makes no representations about the suitability of this
 * software for any purpose.  It is provided "as is" without
 * express or implied warranty.
 *
 * Interface:
 *	Rmt_Init  	    	Initialize things for this module
 *
 *	Rmt_AddServer	    	Add the given name as the address of
 *	    	  	    	an export server.
 *
 *	Rmt_ReExport	    	Re-export a job that has come home to roost.
 *
 *	Rmt_Begin 	    	Prepare to export another job and tell
 *	    	  	    	if it can actually be exported.
 *
 *	Rmt_Exec  	    	Execute the given shell with argument vector
 *	    	  	    	elsewhere.
 *
 *	Rmt_LastID	    	Return an unique identifier for the last
 *	    	  	    	job exported.
 *
 *	Rmt_Done  	    	Take note that a remote job has finished.
 *
 */
#ifndef lint
static char rcsid[] =
"$Header: rmt.c,v 1.9 88/07/23 16:45:40 adam Exp $ SPRITE (Berkeley)";
#endif lint

#include    "make.h"

#include    "customs.h"
#include    <sys/time.h>
#include    <sys/fcntl.h>
#include    <sys/file.h>

static int  	  	exportFD;   /* Socket to server with the most recent
				     * job of ours */
static int  	  	returnFD;   /* Socket on which server will call back */
static ExportPermit	permit;	    /* Permit for last-exported job */
static char 	  	cwd[1024];  /* The current working directory */
static Boolean	  	noAgent;    /* TRUE if agent not present */

/*-
 *-----------------------------------------------------------------------
 * Rmt_Init --
 *	Initialize this module...
 *
 * Results:
 *	None.
 *
 * Side Effects:
 *	The current working directory is placed in cwd and noAgent set
 *	FALSE if the local customs agent could be contacted.
 *
 *-----------------------------------------------------------------------
 */
void
Rmt_Init()
{
    Rpc_Stat	  status;

    if (noExport || ((status = Customs_Ping()) != RPC_SUCCESS)) {
	if (DEBUG(RMT) && !noExport) {
	    printf("Could not contact customs agent: %s\n",
		   Rpc_ErrorMessage(status));
	}
	noAgent = TRUE;
    } else {
	noAgent = FALSE;
	getwd(cwd);
	if (DEBUG(RMT)) {
	    printf("Customs agent present. cwd = \"%s\"\n", cwd);
	}
	exportFD = 0;
    }
}

/*-
 *-----------------------------------------------------------------------
 * Rmt_AddServer --
 *	Add a server to the list of those known.
 *
 * Results:
 *	None.
 *
 * Side Effects:
 *	None.
 *
 *-----------------------------------------------------------------------
 */
/*ARGSUSED*/
void
Rmt_AddServer (name)
    char    *name;
{
}

/*-
 *-----------------------------------------------------------------------
 * Rmt_Begin --
 *	Prepare to export a job.
 *
 * Results:
 *	TRUE if the job can be exported. FALSE if it cannot.
 *
 * Side Effects:
 *	A TCP connection is opened to an available server and the
 *	CUSTOMS_IMPORT command issued (i.e. the job is started
 *	over there). exportFD is set to the fd of the connection and
 *	returnFD to the fd of the socket to be used to return the exit
 *	status.
 *
 *-----------------------------------------------------------------------
 */
/*ARGSUSED*/
Boolean
Rmt_Begin (file, argv, gn)
    char    	  *file;
    char    	  **argv;
    GNode   	  *gn;
{
    if (noAgent) {
	return (FALSE);
    } else {
	extern int errno;
	returnFD = -1;
	errno = 0;

	if (DEBUG(RMT)) {
	    printf("Rmt_Begin: sameArch = %s\n",
		   (gn->type & OP_EXPORTSAME) ? "True" : "False");
	}
	exportFD = Customs_RawExport(file, argv, cwd,
				     ((gn->type & OP_EXPORTSAME) ?
				      EXPORT_SAME :
				      EXPORT_ANY),
				     &returnFD,
				     &permit);
	if (exportFD < 0) {
	    if (DEBUG(RMT)) {
		perror("Customs_RawExport");
		Customs_PError(file);
	    }
	    return (FALSE);
	} else {
	    return (TRUE);
	}
    }
}

/*-
 *-----------------------------------------------------------------------
 * Rmt_ReExport --
 *	Supposed to re-export a job that's come home, but since jobs
 *	can't come home under customs, we just return FALSE to say
 *	we couldn't do it.
 *
 * Results:
 *	FALSE.
 *
 * Side Effects:
 *	None.
 *
 *-----------------------------------------------------------------------
 */
/*ARGSUSED*/
Boolean
Rmt_ReExport(pid)
    int	    pid;
{
    if (DEBUG(RMT)) {
	printf("Rmt_ReExport called?\n");
    }
    return(FALSE);
}

/*-
 *-----------------------------------------------------------------------
 * Rmt_Exec --
 *	Execute a process elsewhere. If the exportation actually succeeded
 *	(exportFD > 0), the "export" program is executed (must be on the
 *	search path) with the -id flag.
 *
 * Results:
 *	None.
 *
 * Side Effects:
 *	That remains to be seen.
 *
 *-----------------------------------------------------------------------
 */
void
Rmt_Exec (file, args, traceMe)
    char    *file;
    char    **args;
    Boolean traceMe;
{
    if (exportFD > 0) {
	char	  fd1[4], fd2[4];
	char	  id[10];
	char	  *argv[6];

	sprintf (fd1, "%d", exportFD);
	sprintf (fd2, "%d", returnFD);
	sprintf (id, "%08x", permit.id);

	argv[0] = "export to";
	argv[1] = "-id";
	argv[2] = fd1;
	argv[3] = fd2;
	argv[4] = id;
	argv[5] = (char *)0;
	if (DEBUG(RMT)) {
	    printf("export -id %s %s %s\n", argv[2], argv[3], argv[4]);
	}
	(void)execvp ("export", argv);
	if (DEBUG(RMT)) {
	    perror("Couldn't exec \"export\"");
	}
    } else {
	if (DEBUG(RMT)) {
	    printf("Rmt_Exec called when exportFD == %d\n", exportFD);
	}
	(void)execvp (file, args);
    }
}

/*-
 *-----------------------------------------------------------------------
 * Rmt_LastID --
 *	Return an unique identifier for the last job exported with Rmt_Exec
 *
 * Results:
 *	Some sort of identifier. Just returns 1.
 *
 * Side Effects:
 *	returnFD and exportFD are closed.
 *
 *-----------------------------------------------------------------------
 */
/*ARGSUSED*/
int
Rmt_LastID(pid)
    int	    	  pid;	    /* PID of job last exported */
{
    (void)close(returnFD);
    (void)close(exportFD);
    exportFD = 0;
    return (1);
}

/*-
 *-----------------------------------------------------------------------
 * Rmt_Done --
 *	Register the completion of a remote job.
 *
 * Results:
 *	None.
 *
 * Side Effects:
 *	None.
 *
 *-----------------------------------------------------------------------
 */
/*ARGSUSED*/
void
Rmt_Done (id)
    int	    id;
{
}
